@@ -82,8 +82,6 @@
 #include "nsCRT.h"
 #include "nsXBLEventHandler.h"
 #include "nsHTMLAtoms.h"
-#include "nsIBoxObject.h"
-#include "nsIDOMNSDocument.h"
 
 static NS_DEFINE_CID(kDOMScriptObjectFactoryCID,
                      NS_DOM_SCRIPT_OBJECT_FACTORY_CID);
@@ -134,8 +132,7 @@ nsXBLPrototypeHandler::nsXBLPrototypeHandler(const PRUnichar* aEvent,
 }
 
 nsXBLPrototypeHandler::nsXBLPrototypeHandler(nsIContent* aHandlerElement)
-  : mBoxObjectForHandlerElement(nsnull),
-    mLineNumber(0),
+  : mLineNumber(0),
     mNextHandler(nsnull),
     mPrototypeBinding(nsnull)
 {
@@ -151,11 +148,8 @@ nsXBLPrototypeHandler::nsXBLPrototypeHandler(nsIContent* aHandlerElement)
 nsXBLPrototypeHandler::~nsXBLPrototypeHandler()
 {
   --gRefCnt;
-  if (mType & NS_HANDLER_TYPE_XUL) {
-    NS_IF_RELEASE(mBoxObjectForHandlerElement);
-  } else if (mHandlerText) {
+  if (!(mType & NS_HANDLER_TYPE_XUL) && mHandlerText)
     nsMemory::Free(mHandlerText);
-  }
 
   // We own the next handler in the chain, so delete it now.
   delete mNextHandler;
@@ -164,15 +158,15 @@ nsXBLPrototypeHandler::~nsXBLPrototypeHandler()
 already_AddRefed<nsIContent>
 nsXBLPrototypeHandler::GetHandlerElement()
 {
-  if (!mBoxObjectForHandlerElement || !(mType & NS_HANDLER_TYPE_XUL)) {
-    return nsnull;
+  nsIContent* result;
+  if (mType & NS_HANDLER_TYPE_XUL) {
+    result = mHandlerElement;
+    NS_IF_ADDREF(result);
   }
-  nsCOMPtr<nsIDOMElement> element;
-  mBoxObjectForHandlerElement->GetElement(getter_AddRefs(element));
-  nsCOMPtr<nsIContent> content = do_QueryInterface(element);
-  nsIContent* handler = nsnull;
-  content.swap(handler);
-  return handler;
+  else
+    result = nsnull;
+
+  return result;
 }
 
 void
@@ -223,16 +217,18 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
   if (mType & NS_HANDLER_TYPE_PREVENTDEFAULT) {
     aEvent->PreventDefault();
     // If we prevent default, then it's okay for
-    // mBoxObjectForHandlerElement and mHandlerText to be null
+    // mHandlerElement and mHandlerText to be null
     rv = NS_OK;
   }
 
-  if (!mBoxObjectForHandlerElement) // This works for both types of handlers.  In both cases, the union's var should be defined.
+  if (!mHandlerElement) // This works for both types of handlers.  In both cases, the union's var should be defined.
     return rv;
 
   // See if our event receiver is a content node (and not us).
   PRBool isXULKey = (mType & NS_HANDLER_TYPE_XUL);
   PRBool isXBLCommand = (mType & NS_HANDLER_TYPE_XBL_COMMAND);
+  NS_ASSERTION(!(isXULKey && isXBLCommand),
+               "can't be both a key and xbl command handler");
 
   // XUL handlers and commands shouldn't be triggered by non-trusted
   // events.
@@ -247,17 +243,12 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
       return NS_OK;
   }
     
-  PRBool isReceiverCommandElement = PR_FALSE;
-  nsCOMPtr<nsIContent> content(do_QueryInterface(aReceiver));
-  nsCOMPtr<nsIContent> handlerElement = GetHandlerElement();
-  if (isXULKey && content && content != handlerElement)
-    isReceiverCommandElement = PR_TRUE;
+  if (isXBLCommand) {
+    // This is a special-case optimization to make command handling fast.
+    // It isn't really a part of XBL, but it helps speed things up.
 
-  // This is a special-case optimization to make command handling fast.
-  // It isn't really a part of XBL, but it helps speed things up.
-  if (isXBLCommand && !isReceiverCommandElement) {
     // See if preventDefault has been set.  If so, don't execute.
-    PRBool preventDefault;
+    PRBool preventDefault = PR_FALSE;
     nsCOMPtr<nsIDOMNSUIEvent> nsUIEvent(do_QueryInterface(aEvent));
     if (nsUIEvent)
       nsUIEvent->GetPreventDefault(&preventDefault);
@@ -324,7 +315,7 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
       nsCOMPtr<nsIContent> focusedContent = do_QueryInterface(focusedElement);
       nsIContent *content = focusedContent;
 
-      // if the focused element is a link then we do want space to
+      // if the focused element is a link then we do want space to 
       // scroll down. focused element may be an element in a link,
       // we need to check the parent node too.
       if (focusedContent) {
@@ -364,6 +355,41 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
     return NS_OK;
   }
 
+  // If we're executing on a XUL key element, just dispatch a command
+  // event at the element.  It will take care of retargeting it to its
+  // command element, if applicable, and executing the event handler.
+  if (isXULKey) {
+    aEvent->PreventDefault();
+
+    nsEventStatus status = nsEventStatus_eIgnore;
+    nsXULCommandEvent event(PR_TRUE, NS_XUL_COMMAND, nsnull);
+
+    // Copy the modifiers from the key event.
+    nsCOMPtr<nsIDOMKeyEvent> keyEvent = do_QueryInterface(aEvent);
+    if (!keyEvent) {
+      NS_ERROR("Trying to execute a key handler for a non-key event!");
+      return NS_ERROR_FAILURE;
+    }
+
+    keyEvent->GetAltKey(&event.isAlt);
+    keyEvent->GetCtrlKey(&event.isControl);
+    keyEvent->GetShiftKey(&event.isShift);
+    keyEvent->GetMetaKey(&event.isMeta);
+    
+    nsPresContext *pc = nsnull;
+    nsIDocument *doc = mHandlerElement->GetCurrentDoc();
+    if (doc) {
+      nsIPresShell *shell = doc->GetShellAt(0);
+      if (shell) {
+        pc = shell->GetPresContext();
+      }
+    }
+
+    mHandlerElement->HandleDOMEvent(pc, &event, nsnull,
+                                    NS_EVENT_FLAG_INIT, &status);
+    return NS_OK;
+  }
+
   // Look for a compiled handler on the element. 
   // Should be compiled and bound with "on" in front of the name.
   nsAutoString onEvent(NS_LITERAL_STRING("onxbl"));
@@ -374,31 +400,6 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
 
   void* handler = nsnull;
   
-  // Compile the event handler.
-  nsAutoString xulText;
-  nsIContent* keyCommandContent = nsnull;
-  if (isXULKey) {
-    // Try an oncommand attribute (used by XUL <key> elements, which
-    // are implemented using this code).
-    NS_ENSURE_STATE(handlerElement);
-    handlerElement->GetAttr(kNameSpaceID_None, nsLayoutAtoms::oncommand, xulText);
-    keyCommandContent = handlerElement;
-    if (xulText.IsEmpty()) {
-      // Maybe the receiver is a <command> elt.
-      if (isReceiverCommandElement) {
-        // It is!  See if it has an oncommand attribute.
-        content->GetAttr(kNameSpaceID_None, nsLayoutAtoms::oncommand, xulText);
-        keyCommandContent = content;
-      }
-      
-      if (xulText.IsEmpty())
-        return NS_ERROR_FAILURE; // For whatever reason, they didn't give us anything to do.
-    }
-  }
-  
-  if (isXULKey)
-    aEvent->PreventDefault(); // Preventing default for XUL key handlers
-
   // Compile the handler and bind it to the element.
   nsCOMPtr<nsIScriptGlobalObject> boundGlobal;
   nsCOMPtr<nsPIWindowRoot> winRoot(do_QueryInterface(aReceiver));
@@ -473,25 +474,16 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
 
   const char *eventName = nsContentUtils::GetEventArgName(kNameSpaceID_XBL);
 
-  if (isXULKey) {
-    nsCAutoString documentURI;
-    keyCommandContent->GetOwnerDoc()->GetDocumentURI()->GetSpec(documentURI);
-    rv = boundContext->CompileEventHandler(scriptObject, onEventAtom, eventName,
-                                           xulText, documentURI.get(), 0, 
-                                           PR_TRUE, &handler);
-  }
-  else {
-    nsDependentString handlerText(mHandlerText);
-    if (handlerText.IsEmpty())
-      return NS_ERROR_FAILURE;
-    
-    nsCAutoString bindingURI;
-    mPrototypeBinding->DocURI()->GetSpec(bindingURI);
-    
-    rv = boundContext->CompileEventHandler(scriptObject, onEventAtom, eventName,
-                                           handlerText, bindingURI.get(),
-                                           mLineNumber, PR_TRUE, &handler);
-  }
+  nsDependentString handlerText(mHandlerText);
+  if (handlerText.IsEmpty())
+    return NS_ERROR_FAILURE;
+  
+  nsCAutoString bindingURI;
+  mPrototypeBinding->DocURI()->GetSpec(bindingURI);
+
+  rv = boundContext->CompileEventHandler(scriptObject, onEventAtom, eventName,
+                                         handlerText, bindingURI.get(),
+                                         mLineNumber, PR_TRUE, &handler);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsAutoGCRoot root(&handler, &rv);
@@ -772,12 +764,7 @@ PRInt32 nsXBLPrototypeHandler::KeyToMask(PRInt32 key)
 void
 nsXBLPrototypeHandler::GetEventType(nsAString& aEvent)
 {
-  nsCOMPtr<nsIContent> handlerElement = GetHandlerElement();
-  if (!handlerElement) {
-    aEvent.Truncate();
-    return;
-  }
-  handlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::event, aEvent);
+  mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::event, aEvent);
   
   if (aEvent.IsEmpty() && (mType & NS_HANDLER_TYPE_XUL))
     // If no type is specified for a XUL <key> element, let's assume that we're "keypress".
@@ -803,17 +790,7 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
 
   if (aKeyElement) {
     mType |= NS_HANDLER_TYPE_XUL;
-
-    nsCOMPtr<nsIDOMNSDocument> nsDomDoc =
-      do_QueryInterface(aKeyElement->GetOwnerDoc());
-    if (nsDomDoc) {
-      nsCOMPtr<nsIDOMElement> el = do_QueryInterface(aKeyElement);
-      nsCOMPtr<nsIBoxObject> box;
-      nsDomDoc->GetBoxObjectFor(el, getter_AddRefs(box));
-      box.swap(mBoxObjectForHandlerElement);
-      NS_WARN_IF_FALSE(mBoxObjectForHandlerElement,
-                       "Couldn't get a box object for the key element!");
-    }
+    mHandlerElement = aKeyElement;
   }
   else {
     mType |= aCommand ? NS_HANDLER_TYPE_XBL_COMMAND : NS_HANDLER_TYPE_XBL_JS;
@@ -863,7 +840,7 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
   // Modifiers are supported by both types of handlers (XUL and XBL).  
   nsAutoString modifiers(aModifiers);
   if (mType & NS_HANDLER_TYPE_XUL)
-    aKeyElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::modifiers, modifiers);
+    mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::modifiers, modifiers);
   
   if (!modifiers.IsEmpty()) {
     mKeyMask = cAllModifiers;
@@ -895,9 +872,9 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
   nsAutoString key(aCharCode);
   if (key.IsEmpty()) {
     if (mType & NS_HANDLER_TYPE_XUL) {
-      aKeyElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::key, key);
+      mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::key, key);
       if (key.IsEmpty()) 
-        aKeyElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::charcode, key);
+        mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::charcode, key);
     }
   }
 
@@ -918,7 +895,7 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
   else {
     key.Assign(aKeyCode);
     if (mType & NS_HANDLER_TYPE_XUL)
-      aKeyElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::keycode, key);
+      mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::keycode, key);
     
     if (!key.IsEmpty()) {
       if (mKeyMask == 0)
