@@ -592,28 +592,43 @@ nsresult CTextToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt32 aFlag)
     if(NS_OK==result) {
       result=aScanner.Peek(aChar);
 
-      if(((kCR==aChar) || (kNewLine==aChar)) && (NS_OK==result)) {
-        result=aScanner.GetChar(aChar); //strip off the char
-        PRUnichar theNextChar;
-        result=aScanner.Peek(theNextChar);    //then see what's next.
-        switch(aChar) {
+      if (NS_OK == result && (kCR == aChar || kNewLine == aChar)) {
+        switch (aChar) {
           case kCR:
-            // result=aScanner.GetChar(aChar);       
-            if(kLF==theNextChar) {
-              // If the "\r" is followed by a "\n", don't replace it and 
-              // let it be ignored by the layout system
+          {
+            // It's a carriage return. See if this is part of a CR-LF pair (in
+            // which case we need to treat it as one newline). If we're at the
+            // edge of a packet, then leave the CR on the scanner, since it
+            // could still be part of a CR-LF pair. Otherwise, it isn't.
+            PRUnichar theNextChar;
+            result = aScanner.Peek(theNextChar, 1);
+
+            if (result == kEOF && aScanner.IsIncremental()) {
+              break;
+            }
+
+            if (NS_SUCCEEDED(result)) {
+              // Actually get the carriage return.
+              aScanner.GetChar(aChar);
+            }
+
+            if (kLF == theNextChar) {
+              // If the "\r" is followed by a "\n", don't replace it and let
+              // it be ignored by the layout system.
               end.advance(2);
-              result=aScanner.GetChar(theNextChar);
+              aScanner.GetChar(theNextChar);
             }
             else {
-              // If it standalone, replace the "\r" with a "\n" so that 
-              // it will be considered by the layout system
+              // If it is standalone, replace the "\r" with a "\n" so that it
+              // will be considered by the layout system.
               aScanner.ReplaceCharacter(end, kLF);
               ++end;
             }
             ++mNewlineCount;
             break;
+          }
           case kLF:
+            aScanner.GetChar(aChar);
             ++end;
             ++mNewlineCount;
             break;
@@ -622,7 +637,10 @@ nsresult CTextToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt32 aFlag)
       else done=PR_TRUE;
     }
   }
-  
+
+  // Note: This function is only called from nsHTMLTokenizer::ConsumeText. If
+  // we return an error result from the final buffer, then it is responsible
+  // for turning it into an NS_OK result.
   aScanner.BindSubstring(mTextValue, origin, end);
 
   return result;
@@ -815,7 +833,7 @@ nsresult CTextToken::ConsumeParsedCharacterData(PRBool aDiscardFirstNewline,
   // the document.
 
   static const PRUnichar terminalChars[] = {
-    PRUnichar('&'), PRUnichar('<'),
+    PRUnichar('\r'), PRUnichar('\n'), PRUnichar('&'), PRUnichar('<'),
     PRUnichar(0)
   };
   static const nsReadEndCondition theEndCondition(terminalChars);
@@ -1723,6 +1741,41 @@ PRInt32 CAttributeToken::GetTokenType(void) {
   return eToken_attribute;
 }
 
+/*
+ *  Removes non-alpha-non-digit characters from the end of a KEY
+ *  
+ *  @update harishd 07/15/99
+ *  @param  
+ *  @return  
+ */
+void CAttributeToken::SanitizeKey() {
+  PRInt32   length=mTextKey.Length();
+  if(length > 0) {
+    nsScannerIterator iter, begin, end;
+    mTextKey.BeginReading(begin);
+    mTextKey.EndReading(end);
+    iter = end;
+
+    // Look for the first legal character starting from
+    // the end of the string
+    do {
+      --iter;
+    } while (!nsCRT::IsAsciiAlpha(*iter) && 
+             !nsCRT::IsAsciiDigit(*iter) && 
+             (iter != begin));
+    
+    // If there were any illegal characters, just copy out the
+    // legal part
+    if (iter != --end) {
+      nsAutoString buf;
+      CopyUnicodeTo(begin, ++iter, buf);
+      mTextKey.Rebind(buf);
+    }
+  }
+
+  return;
+}
+
 const nsSubstring& CAttributeToken::GetStringValue(void)
 {
   return mTextValue.str();
@@ -1824,7 +1877,7 @@ nsresult ConsumeQuotedString(PRUnichar aChar,
 /*
  * This method is meant to be used by view-source to consume invalid attributes.
  * For the purposes of this method, an invalid attribute is an attribute that
- * starts with either ', ", or /. We consume all ', ", or / and the following whitespace.
+ * starts with either ' or ". We consume all ' or " and the following whitespace.
  * 
  * @param aScanner -- the scanner we're reading our data from.
  * @param aChar -- the character we're skipping
@@ -1837,30 +1890,31 @@ nsresult ConsumeInvalidAttribute(nsScanner& aScanner,
                                  PRUnichar aChar,
                                  nsScannerIterator& aCurrent,
                                  PRInt32& aNewlineCount) {
-  NS_ASSERTION(aChar == kApostrophe || aChar == kQuote || aChar == kForwardSlash,
-               "aChar must be a quote or apostrophe");
+  NS_ASSERTION(aChar=='\'' || aChar=='"', "aChar must be a quote or apostrophe");
   nsScannerIterator end, wsbeg;
   aScanner.EndReading(end);
 
-  while (aCurrent != end && *aCurrent == aChar) {
+  while (aCurrent!=end && *aCurrent==aChar) {
     ++aCurrent;
   }
 
   aScanner.SetPosition(aCurrent);
-  return aScanner.ReadWhitespace(wsbeg, aCurrent, aNewlineCount);
+  return aScanner.ReadWhitespace(wsbeg,aCurrent,aNewlineCount);
 }
 
 /*
  *  Consume the key and value portions of the attribute.
  *  
+ *  @update  rickg 03.23.2000
  *  @param   aChar -- last char consumed from stream
  *  @param   aScanner -- controller of underlying input source
  *  @param   aFlag - contains information such as |dtd mode|view mode|doctype|etc...
  *  @return  error result
  */
-nsresult CAttributeToken::Consume(PRUnichar aChar, nsScanner& aScanner, PRInt32 aFlag)
-{
+nsresult CAttributeToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt32 aFlag) {
+
   nsresult result;
+ 
   nsScannerIterator wsstart, wsend;
   
   if (aFlag & NS_IPARSER_FLAG_VIEW_SOURCE) {
@@ -1882,7 +1936,7 @@ nsresult CAttributeToken::Consume(PRUnichar aChar, nsScanner& aScanner, PRInt32
       PRUnichar('\r'), PRUnichar('\t'), 
       PRUnichar('>'), PRUnichar('<'),
       PRUnichar('\b'), PRUnichar('\''),
-      PRUnichar('/'), PRUnichar(0) };
+      PRUnichar(0) };
     static const nsReadEndCondition theEndCondition(theTerminalsChars);
 
     nsScannerIterator start, end;
@@ -1984,31 +2038,25 @@ nsresult CAttributeToken::Consume(PRUnichar aChar, nsScanner& aScanner, PRInt32
           else {
             //This is where we have to handle fairly busted content.
             //If you're here, it means we saw an attribute name, but couldn't find 
-            //the following equal sign.  <tag NAME....
+            //the following equal sign.  <tag NAME=....
         
             //Doing this right in all cases is <i>REALLY</i> ugly. 
             //My best guess is to grab the next non-ws char. We know it's not '=',
             //so let's see what it is. If it's a '"', then assume we're reading
             //from the middle of the value. Try stripping the quote and continuing...
-            //Note that this code also strips forward slashes to handle cases
-            //like <tag NAME/>
-            if (kQuote == aChar || kApostrophe == aChar ||
-                kForwardSlash == aChar) {
-              // In XML, a trailing slash isn't an error.
-              if (kForwardSlash != aChar || !(aFlag & NS_IPARSER_FLAG_XML)) {
-                mInError = PR_TRUE;
-              }
+            if (kQuote==aChar || kApostrophe==aChar){
+              mInError=PR_TRUE;
 
               if (!(aFlag & NS_IPARSER_FLAG_VIEW_SOURCE)) {
-                result = aScanner.SkipOver(aChar); // Strip quote or slash.
+                result=aScanner.SkipOver(aChar); //strip quote.
                 if (NS_SUCCEEDED(result)) {
-                  result = aScanner.SkipWhitespace(mNewlineCount);
+                  result=aScanner.SkipWhitespace(mNewlineCount);
                 }
               } else {
                 //We want to collect whitespace here so that following 
                 //attributes can have the right line number (and for
                 //parity with the non-view-source code above).
-                result = ConsumeInvalidAttribute(aScanner, aChar, wsend, mNewlineCount);
+                result=ConsumeInvalidAttribute(aScanner,aChar,wsend,mNewlineCount);
 
                 aScanner.BindSubstring(mTextKey, wsstart, wsend);
                 aScanner.SetPosition(wsend);
